/*
 * time.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vdr/plugin.h>
#include <vdr/thread.h>
#include <vdr/osd.h>
#include "i18n.h"
#include "enhancedbitmap.h"
#include <vdr/menuitems.h>
#include <math.h>
static const char *VERSION        = "0.0.2";
static const char *DESCRIPTION    = "A Simple Clock";
static const char *MAINMENUENTRY  = "Time";

// Global variables

int       TimeWidth  = Setup.OSDwidth;
int       TimeHeight = Setup.OSDheight;
int       XOffset    = 44;
int       YOffset    = 0;
int       Size       = 2;
int       ShowSec    = true;
int       FullScreen = false;
int       ShowAnalog = false;
int       SW         = 1;
int       SL         = 99;
int       MW         = 5;
int       ML         = 90;
int       HW         = 10;
int       HL         = 50;
double    SecWidth   = ((double)SW) / 100;
double    SecLength  = ((double)SL) / 100;
double    MinWidth   = ((double)MW) / 100;
double    MinLength  = ((double)ML) / 100;
double    HourWidth  = ((double)HW) / 100;
double    HourLength = ((double)HL) / 100;
int       ColorBG    = 0;
int       ColorFG    = 1;
int       ColorHour  = 1;
int       ColorMin   = 1;
int       ColorSec   = 3;
char      *TimeColors[]  = {"Transparent",
                            "White",
	                    "Black",
			    "Red",
			    "Green",
			    "Yellow",
			    "Blue",
                           };

class cOSDTime : public cThread, public cOsdObject{
private:
  eKeys LastKey;
  bool running, shutdown;
  eDvbColor ColorH, ColorM, ColorS, ColorForeGround, ColorBackGround;
  int r;

  void DrawArrow(double width, double len, eDvbColor col, double ang, cEnhancedBitmap &img);
  void DrawScale(int r, eDvbColor col, cEnhancedBitmap &Analog);
  void DrawPoint(double x, double y, int r, eDvbColor col, cEnhancedBitmap &Analog);
  eDvbColor getColor(int color);

protected:
  virtual void Action(void);
public:
  cOSDTime(void);
  virtual ~cOSDTime(void);
  void Show(void) {Start(); }
  eOSState ProcessKey(eKeys Key);
};

cOSDTime::cOSDTime()
{
  running = true;
  r = Size*16;
}

cOSDTime::~cOSDTime()
{
  running = false;
  while (!shutdown)
    {
      usleep(100000);
      running = false;
    }
}

void cOSDTime::DrawArrow(double width, double len, eDvbColor col, double ang, cEnhancedBitmap &img)
{
  double s=sin(ang);
  double c=cos(ang);
  img.FillTriangle(r-r*width*c,r-r*width*s,r+r*width*c,r+r*width*s,r+r*len*s,r-r*len*c,col);
  img.FillTriangle(r-r*width*c,r-r*width*s,r+r*width*c,r+r*width*s,r-r*width*s,r+r*width*c,col);
}

void cOSDTime::DrawPoint(double x, double y, int r, eDvbColor col, cEnhancedBitmap &Analog)
{
  double ang = 2*3.1415/20;
  for (int i = 0; i < 20; i++)
    {
      double s=sin(ang * i);
      double c=cos(ang * i);
      Analog.DrawLine(x, y, x + r*0.02*s, y + r*0.02*c, col);
    }
}

void cOSDTime::DrawScale(int r, eDvbColor col, cEnhancedBitmap &Analog)
{

  /*
   * Zeitraster zeichnen
   */
  double ang = 2*3.1415*5/60;
  for (int i = 0; i < 12; i++)
    {
      double s=sin(ang * i);
      double c=cos(ang * i);
      DrawPoint(r+r*s, r+r*c, r, col, Analog);
      if (i % 3 == 0)
	{
	  for (int j = 0; j < 10; j++)
	    {
	      DrawPoint(r+r*s-0.1*r*s*j/10, r+r*c-0.1*r*c*j/10, r, col, Analog);
	    }
	}
    }
}

eDvbColor cOSDTime::getColor(int color)
{
  switch (color)
    {
    case 0  : return clrTransparent;
    case 1  : return clrWhite;
    case 2  : return clrBlack;
    case 3  : return clrRed;
    case 4  : return clrGreen;
    case 5  : return clrYellow;
    case 6  : return clrBlue;
    default : return clrBlack;
    }
  return clrBlack;
}

void cOSDTime::Action(void)
{
  //  int Width, i, ToClear;
  time_t Result;
  char TimeString[250];
  double SecAng, SecAng2,MinAng,HourAng;
  struct tm *CurTime;
  shutdown = false;

  if (FullScreen)
      r = 12 * (TimeHeight-1);
  int ClockSize = 2 * r + 1;
  cEnhancedBitmap Analog(ClockSize,ClockSize,6);

  /*
   * Farben initialisieren
   */

  ColorBackGround = getColor(ColorBG);
  ColorForeGround = getColor(ColorFG);
  ColorH          = getColor(ColorHour);
  ColorM          = getColor(ColorHour);
  ColorS          = getColor(ColorSec);

  Analog.SetColor(0,ColorBackGround);
  Analog.SetColor(1,ColorForeGround);
  Analog.SetColor(2,ColorH);
  Analog.SetColor(3,ColorM);
  Analog.SetColor(4,ColorS);


  if (Interface->IsOpen())
    {
      Interface->Close();
    }
  Interface->Open(TimeWidth, TimeHeight);
  Interface->ClearEol(3, 0, clrBlue);
  for (int i = 0; i < TimeHeight; i++)
    {
      Interface->Write(0,i,"X", clrBlack, clrTransparent);
      Interface->ClearEol(0, i, clrTransparent);
    }

  while (running)
    {
      Result = time(NULL);
      CurTime = localtime(&Result);
      if (ShowSec)
	sprintf(TimeString, " %02d:%02d:%02d ", CurTime->tm_hour, CurTime->tm_min, CurTime->tm_sec);
      else
	sprintf(TimeString, " %02d:%02d ", CurTime->tm_hour, CurTime->tm_min);
      if (ShowAnalog || FullScreen)
	{
	  SecAng=2*3.1415*CurTime->tm_sec/60;
	  SecAng2=2*3.1415*(CurTime->tm_sec-1)/60;
	  MinAng=2*3.1415*((double)CurTime->tm_min+((double)CurTime->tm_sec)/60)/60;
	  HourAng=2*3.1415*((double)CurTime->tm_hour+((double)CurTime->tm_min+((double)CurTime->tm_sec)/60)/60)/12;
	  Analog.DrawLine(0,0,1,1, ColorBackGround);
	  DrawScale(r, ColorForeGround, Analog);
	  DrawArrow(HourWidth,HourLength,ColorH,HourAng,Analog);
	  DrawArrow(MinWidth,MinLength,ColorM,MinAng,Analog);
	  if (ShowSec)
	    DrawArrow(SecWidth,SecLength,ColorS,SecAng,Analog);
	  if (FullScreen)
	    Interface->SetBitmap(((TimeWidth*8 - r) / 2), 2*14, Analog);
	  else
	    Interface->SetBitmap(XOffset * 12, (YOffset + 2)*14, Analog);

	  if (ShowSec)
	    {
	      DrawArrow(SecWidth,SecLength,ColorBackGround,SecAng,Analog);
	      DrawArrow(SecWidth,SecLength,ColorBackGround,SecAng2,Analog);
	    }
	  DrawArrow(MinWidth,MinLength,ColorBackGround,MinAng,Analog);
	  DrawArrow(HourWidth,HourLength,ColorBackGround,HourAng,Analog);
	  Analog.Clean();
	}
      else
	{
	  Interface->Write(XOffset, YOffset + 1, TimeString, ColorForeGround, ColorBackGround);
	}
      usleep(500000);
    } // while running
  shutdown = true;
  Interface->Close();
  cRemote::Put(LastKey);
}

eOSState cOSDTime::ProcessKey(eKeys Key)
{
  if (Key != kNone)
    {
      LastKey = Key;
      return (osEnd);
    }
  return (osContinue);
}

class cPluginTime : public cPlugin {
private:
  // Add any member variables or functions you may need here.
public:
  cPluginTime(void);
  virtual ~cPluginTime();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Start(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  };

// --- cMenuSetupTime -------------------------------------------------------

class cMenuSetupTime : public cMenuSetupPage {
private:
  char Color[16][32];
  int newSize;
  int newShowSec;
  int newFullScreen;
  int newShowAnalog;
  int newHW;
  int newHL;
  int newMW;
  int newML;
  int newSW;
  int newSL;
  int newColorBG;
  int newColorFG;
  int newColorHour;
  int newColorMin;
  int newColorSec;
  int newXOffset;
  int newYOffset;
  int x;
  int y;
protected:
  virtual void Store(void);
public:
  cMenuSetupTime(void);
  };

cMenuSetupTime::cMenuSetupTime(void)
{
  cSetup data;
  newSize       = Size;
  newShowSec    = ShowSec;
  newFullScreen = FullScreen;
  newShowAnalog = ShowAnalog;
  newHW         = HW;
  newHL         = HL;
  newMW         = MW;
  newML         = ML;
  newSW         = SW;
  newSL         = SL;
  newColorBG    = ColorBG;
  newColorFG    = ColorFG;
  newColorHour  = ColorHour;
  newColorMin   = ColorMin;
  newColorSec   = ColorSec;
  newXOffset    = XOffset;
  newYOffset    = YOffset;

  if (!ShowAnalog)
    {
      y = 2;
      if (ShowSec)
	x = 8;
      else
	x = 6;
    }
  else
    {
      x = 3 * Size;
      y = 0;

    }


  Add(new cMenuEditIntItem(  tr("x Position"),               &newXOffset,    0, Setup.OSDwidth - x));
  Add(new cMenuEditIntItem(  tr("y Position"),               &newYOffset,    0, Setup.OSDheight - y));
  Add(new cMenuEditBoolItem( tr("Show Seconds"),             &newShowSec              ));
  Add(new cMenuEditBoolItem( tr("FullScreen"),               &newFullScreen           ));
  Add(new cMenuEditBoolItem( tr("Show analog clock"),        &newShowAnalog           ));
  Add(new cMenuEditIntItem(  tr("Analog clock size"),        &newSize,       0, 15    ));
  Add(new cMenuEditStraItem( tr("Background color"),         &newColorBG,    7, TimeColors));
  Add(new cMenuEditStraItem( tr("Foreground color"),         &newColorFG,    7, TimeColors));
  //  Add(new cMenuEditStraItem( tr("Hour arrow color"),          &newColorHour,  7, TimeColors));
  Add(new cMenuEditStraItem( tr("Arrow color"),              &newColorHour,  7, TimeColors));
  Add(new cMenuEditIntItem(  tr("Length of hour arrow [%]"), &newHL,         0, 100   ));
  Add(new cMenuEditIntItem(  tr("Width of hour arrow [%]"),  &newHW,         0, 100   ));
  //  Add(new cMenuEditStraItem( tr("Minutes arrow color"),      &newColorMin,   7, TimeColors));
  Add(new cMenuEditIntItem(  tr("Length of min arrow [%]"),  &newML,         0, 100   ));
  Add(new cMenuEditIntItem(  tr("Width of min arrow [%]"),   &newMW,         0, 100   ));
  Add(new cMenuEditStraItem( tr("Seconds arrow color"),      &newColorSec,   7, TimeColors));
  Add(new cMenuEditIntItem(  tr("Length of sec arrow [%]"),  &newSL,         0, 100   ));
  Add(new cMenuEditIntItem(  tr("Width of sec arrow [%]"),   &newMW,         0, 100   ));
}

void cMenuSetupTime::Store(void)
{
  SetupStore("XOffset",    XOffset    = newXOffset);
  SetupStore("YOffset",    YOffset    = newYOffset);
  SetupStore("ShowSec",    ShowSec    = newShowSec);
  SetupStore("FullScreen", FullScreen = newFullScreen);
  SetupStore("ShowAnalog", ShowAnalog = newShowAnalog);
  SetupStore("ColorBG",    ColorBG    = newColorBG);
  SetupStore("ColorFG",    ColorFG    = newColorFG);
  SetupStore("ColorHour",  ColorHour  = newColorHour);
  SetupStore("ColorSec",   ColorSec   = newColorSec);
  SetupStore("ColorMin",   ColorMin   = newColorMin);
  SetupStore("Size",       Size       = newSize);
  SetupStore("HW",         HW         = newHW);
  SetupStore("HL",         HL         = newHL);
  SetupStore("MW",         MW         = newMW);
  SetupStore("ML",         ML         = newML);
  SetupStore("SW",         SW         = newSW);
  SetupStore("SL",         SL         = newSL);
}
// --- End cMenuSetupTime


cPluginTime::cPluginTime(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginTime::~cPluginTime()
{
  // Clean up after yourself!
}

const char *cPluginTime::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return tr("This Plugin has no Options for the Command Line");
}

bool cPluginTime::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginTime::Start(void)
{
  // Start any background activities the plugin shall perform.
  RegisterI18n(Phrases);
  return true;
}

void cPluginTime::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

cOsdObject *cPluginTime::MainMenuAction(void)
{
  return new cOSDTime();
}

cMenuSetupPage *cPluginTime::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return new cMenuSetupTime;
}

bool cPluginTime::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.

  if      (!strcasecmp(Name, "XOffset"))          XOffset    = atoi(Value);
  else if (!strcasecmp(Name, "YOffset"))          YOffset    = atoi(Value);
  else if (!strcasecmp(Name, "ShowSec"))          ShowSec    = atoi(Value);
  else if (!strcasecmp(Name, "FullScreen"))       FullScreen = atoi(Value);
  else if (!strcasecmp(Name, "ShowAnalog"))       ShowAnalog = atoi(Value);
  else if (!strcasecmp(Name, "Size"))             Size       = atoi(Value);
  else if (!strcasecmp(Name, "ColorBG"))          ColorBG    = atoi(Value);
  else if (!strcasecmp(Name, "ColorFG"))          ColorFG    = atoi(Value);
  else if (!strcasecmp(Name, "ColorHour"))        ColorHour  = atoi(Value);
  else if (!strcasecmp(Name, "ColorMin"))         ColorMin   = atoi(Value);
  else if (!strcasecmp(Name, "ColorSec"))         ColorSec   = atoi(Value);
  else if (!strcasecmp(Name, "HW"))               HW         = atoi(Value);
  else if (!strcasecmp(Name, "HL"))               HL         = atoi(Value);
  else if (!strcasecmp(Name, "MW"))               MW         = atoi(Value);
  else if (!strcasecmp(Name, "ML"))               ML         = atoi(Value);
  else if (!strcasecmp(Name, "SW"))               SW         = atoi(Value);
  else if (!strcasecmp(Name, "SL"))               SL         = atoi(Value);
  else
     return false;
  return true;
}

VDRPLUGINCREATOR(cPluginTime); // Don't touch this!

/*  LocalWords:  plugin README vdr const
 */
